@@ -651,6 +651,14 @@ extern "C" {
     JNIEXPORT jstring JNICALL Java_org_iotivity_base_OcRepresentation_getUri
         (JNIEnv *, jobject);
 
+    /*
+    * Class:     org_iotivity_base_OcRepresentation
+    * Method:    getHost
+    * Signature: ()Ljava/lang/String;
+    */
+    JNIEXPORT jstring JNICALL Java_org_iotivity_base_OcRepresentation_getHost
+        (JNIEnv *, jobject);
+
     /*
     * Class:     org_iotivity_base_OcRepresentation
     * Method:    setUri
