@@ -35,15 +35,15 @@ namespace OIC
 
 class JNIEnvWrapper;
 
-void initJavaExceptions(JNIEnvWrapper *);
-void clearJavaExceptions(JNIEnvWrapper *);
+void initJavaExceptions(JNIEnvWrapper*);
+void clearJavaExceptions(JNIEnvWrapper*);
 
-void throwPlatformException(JNIEnv *, const OIC::Service::RCSPlatformException &);
+void throwPlatformException(JNIEnv*, const OIC::Service::RCSPlatformException&);
 
 template < typename ENV >
-void throwRCSException(ENV *env, const char *msg)
+void throwRCSException(ENV* env, const char* msg)
 {
-    env->ThrowNew(env->FindClass(EXC_NAME_RCS), msg);
+    //env->ThrowNew(env->FindClass(EXC_NAME_RCS), msg);
 }
 
 
