@@ -528,18 +528,6 @@ void *showContainerAPIs(void *data)
         elm_list_item_append(listnew, "4. Remove HUE Bundle Resource", NULL, NULL,
                              removeHueResourceConfig, NULL);
 
-        elm_list_item_append(listnew, "5. Add BMI Bundle", NULL, NULL,
-                             addBMIBundle, NULL);
-
-        elm_list_item_append(listnew, "6. Start BMI Bundle", NULL, NULL,
-                             startBMIBundle, NULL);
-
-        elm_list_item_append(listnew, "7. Remove BMI Bundle", NULL, NULL,
-                             removeBMIBundle, NULL);
-
-        elm_list_item_append(listnew, "8. Stop BMI Bundle", NULL, NULL,
-                             stopBMIBundle, NULL);
-
         elm_list_go(listnew);
     }
     return NULL;
@@ -563,7 +551,7 @@ static void startContainer(void *data, Evas_Object *obj, void *event_info)
             s_containerFlag = true;
             s_hueBundleFlag = true;
             logMessage += "CONTAINER STARTED<br>";
-            logMessage += "ADD AND START BUNDLES<br>";
+            logMessage += "HUE BUNDLE STARTED<br>";
         }
         else
         {
@@ -593,8 +581,8 @@ static void stopContainer(void *data, Evas_Object *obj, void *event_info)
     if (checkContainer)
     {
         s_containerFlag = false;
-        removeHueBundle(NULL, NULL, NULL);
         stopHueBundle(NULL, NULL, NULL);
+        removeHueBundle(NULL, NULL, NULL);
 
         container->stopContainer();
         logMessage += "CONTAINER STOPPED<br>";
@@ -625,13 +613,14 @@ naviframe_pop_cb(void *data, Elm_Object_Item *it)
         evas_object_del(listnew);
         listnew = NULL;
     }
+
+    ui_app_exit();
     return EINA_TRUE;
 }
 
 // Method to set up server screens
-void containerCreateUI(void *data, Evas_Object *obj, void *event_info)
+void containerCreateUI(void *data)
 {
-	dlog_print(DLOG_INFO, LOG_TAG, "#### container UI");
     s_containerFlag = false;
     s_hueBundleFlag = false;
     Evas_Object *layout;
@@ -646,7 +635,6 @@ void containerCreateUI(void *data, Evas_Object *obj, void *event_info)
     scroller = elm_scroller_add(nf);
     elm_scroller_bounce_set(scroller, EINA_FALSE, EINA_TRUE);
     elm_scroller_policy_set(scroller, ELM_SCROLLER_POLICY_OFF, ELM_SCROLLER_POLICY_AUTO);
-    dlog_print(DLOG_INFO, LOG_TAG, "#### container UI 1");
 
     // Layout
     layout = elm_layout_add(nf);
@@ -654,21 +642,18 @@ void containerCreateUI(void *data, Evas_Object *obj, void *event_info)
     evas_object_size_hint_weight_set(layout, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
 
     elm_object_content_set(scroller, layout);
-    dlog_print(DLOG_INFO, LOG_TAG, "#### container UI 2");
 
     // Button
     start_button = elm_button_add(layout);
     elm_object_part_content_set(layout, "start_button", start_button);
     elm_object_text_set(start_button, "Start Container");
     evas_object_smart_callback_add(start_button, "clicked", startContainer, NULL);
-    dlog_print(DLOG_INFO, LOG_TAG, "#### container UI 3");
 
     // Button
     stop_button = elm_button_add(layout);
     elm_object_part_content_set(layout, "stop_button", stop_button);
     elm_object_text_set(stop_button, "Stop Container");
     evas_object_smart_callback_add(stop_button, "clicked", stopContainer, NULL);
-    dlog_print(DLOG_INFO, LOG_TAG, "#### container UI 4");
 
     // List
     listnew = elm_list_add(layout);
@@ -676,7 +661,6 @@ void containerCreateUI(void *data, Evas_Object *obj, void *event_info)
     evas_object_smart_callback_add(listnew, "selected", list_selected_cb, NULL);
     elm_object_part_content_set(layout, "listnew", listnew);
     elm_list_go(listnew);
-    dlog_print(DLOG_INFO, LOG_TAG, "#### container UI 5");
 
     // log_entry - text area for log
     log_entry = elm_entry_add(layout);
@@ -687,7 +671,6 @@ void containerCreateUI(void *data, Evas_Object *obj, void *event_info)
     evas_object_size_hint_align_set(log_entry, EVAS_HINT_FILL, EVAS_HINT_FILL);
     elm_object_part_content_set(layout, "log", log_entry);
 
-    nf_it = elm_naviframe_item_push(nf, "Resource Container", NULL, NULL, scroller, NULL);
+    nf_it = elm_naviframe_item_push(nf, "Container Server", NULL, NULL, scroller, NULL);
     elm_naviframe_item_pop_cb_set(nf_it, naviframe_pop_cb, NULL);
-    dlog_print(DLOG_INFO, LOG_TAG, "#### container UI 6");
 }
