@@ -21,9 +21,20 @@
 #ifndef RECLIENT_H__
 #define RECLIENT_H__
 
+#include <string>
+
 typedef void(*ClientMenuHandler)();
 typedef int ReturnValue;
 
+const std::string TEMPERATURE_URI = "/a/TempSensor";
+const std::string LIGHT_URI = "/a/light";
+const std::string TEMPERATURE_RT = "oic.r.temperaturesensor";
+const std::string LIGHT_RT = "oic.r.light";
+const std::string TEMPERATURE_AK = "Temperature";
+const std::string LIGHT_AK = "Brightness";
+
+void client_cb(void *data);
+
 void *showClientAPIs(void *data);
 
 #endif // RECLIENT_H__
\ No newline at end of file
