@@ -16,24 +16,6 @@
 
 package oic.simulator.clientcontroller.view;
 
-import java.io.BufferedWriter;
-import java.io.FileWriter;
-import java.io.IOException;
-import java.text.DateFormat;
-import java.text.SimpleDateFormat;
-import java.util.HashMap;
-import java.util.List;
-
-import oic.simulator.clientcontroller.Activator;
-import oic.simulator.clientcontroller.listener.ILogUIListener;
-import oic.simulator.clientcontroller.manager.LogManager;
-import oic.simulator.clientcontroller.utils.Constants;
-import oic.simulator.clientcontroller.view.dialogs.FilterDialog;
-import oic.simulator.clientcontroller.view.dialogs.LogDetailsDialog;
-import oic.simulator.logger.LogContentProvider;
-import oic.simulator.logger.LogEntry;
-import oic.simulator.logger.LogLabelProvider;
-
 import org.eclipse.jface.action.Action;
 import org.eclipse.jface.action.IAction;
 import org.eclipse.jface.action.IContributionItem;
@@ -72,8 +54,29 @@ import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.dialogs.FilteredTree;
 import org.eclipse.ui.dialogs.PatternFilter;
 import org.eclipse.ui.part.ViewPart;
+
+import java.io.BufferedWriter;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.OutputStreamWriter;
+import java.text.DateFormat;
+import java.text.SimpleDateFormat;
+import java.util.Date;
+import java.util.HashMap;
+import java.util.List;
+
 import org.oic.simulator.ILogger.Level;
 
+import oic.simulator.clientcontroller.Activator;
+import oic.simulator.clientcontroller.listener.ILogUIListener;
+import oic.simulator.clientcontroller.manager.LogManager;
+import oic.simulator.clientcontroller.utils.Constants;
+import oic.simulator.clientcontroller.view.dialogs.FilterDialog;
+import oic.simulator.clientcontroller.view.dialogs.LogDetailsDialog;
+import oic.simulator.logger.LogContentProvider;
+import oic.simulator.logger.LogEntry;
+import oic.simulator.logger.LogLabelProvider;
+
 /**
  * This class manages and shows the log view in the perspective.
  */
@@ -429,9 +432,10 @@ public class LogView extends ViewPart {
                 String data = sb.toString();
                 BufferedWriter out = null;
                 try {
-                    out = new BufferedWriter(new FileWriter(name));
+                    out = new BufferedWriter(new OutputStreamWriter(
+                            new FileOutputStream(name), "UTF-8"));
                     out.write(data);
-                } catch (IOException e) {
+                } catch (Exception e) {
                     e.printStackTrace();
                     MessageDialog.openError(
                             Display.getDefault().getActiveShell(),
@@ -444,7 +448,13 @@ public class LogView extends ViewPart {
                             out.close();
                         }
                     } catch (IOException e) {
-                        System.out.println("Error occurred during close.");
+                        Activator
+                                .getDefault()
+                                .getLogManager()
+                                .log(Level.ERROR.ordinal(),
+                                        new Date(),
+                                        "[" + e.getClass().getSimpleName()
+                                                + "]" + e.getMessage());
                     }
                 }
             }
